
#include <windows.h>  
#include "timecli.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "Time Client"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

HDDEDATA CALLBACK DdemlCallback( UINT     uType,
                                 UINT     uFmt,	
                                 HCONV    hconv,	
                                 HSZ      hsz1,	
                                 HSZ      hsz2,	
                                 HDDEDATA hdata,	
                                 DWORD    dwData1,
                                 DWORD    dwData2 );


DWORD     ddeInst    = 0;
HSZ       hszService = NULL;
HSZ       hszTopic   = NULL;
HSZ       hszItem    = NULL;
HCONV     hConv      = NULL;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static BOOL bAdv  = FALSE;
static HWND hList = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              // Initialize DDEML.
              //..................
              if ( DdeInitialize( &ddeInst, DdemlCallback, 
                   APPCMD_CLIENTONLY, 0 ) != DMLERR_NO_ERROR )
              {
                 MessageBox( hWnd, "Could not initialize DDEML.", 
                             NULL, MB_OK | MB_ICONSTOP );
                 return( -1 );
              }

              // Allocate strings.
              //..................
              hszService = DdeCreateStringHandle( ddeInst,"Server",CP_WINANSI );
              hszTopic   = DdeCreateStringHandle( ddeInst,"Time",CP_WINANSI );
              hszItem    = DdeCreateStringHandle( ddeInst,"TimeItem",CP_WINANSI );

              hList = CreateWindow( "LISTBOX", "",    
                                    WS_CHILD | LBS_STANDARD | 
                                    WS_VISIBLE | LBS_NOINTEGRALHEIGHT, 
                                    0, 0, 
                                    0, 0,  
                                    hWnd,              
                                    (HMENU)101,              
                                    hInst,         
                                    NULL               
                                  );

              // Connect to server.
              //...................
              hConv = DdeConnect( ddeInst, hszService, hszTopic, NULL );

              if ( DdeGetLastError( ddeInst ) != DMLERR_NO_ERROR )
              {
                 MessageBox( hWnd, "Could not connect to server.", 
                                   NULL, MB_OK | MB_ICONSTOP );
              }
              else
                 DdeSetUserHandle( hConv, QID_SYNC, (DWORD)hList );
              break;

      case WM_SIZE :
              MoveWindow( hList, 0, 0, LOWORD( lParam ), HIWORD( lParam ), TRUE );
              break; 

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        if ( hConv && !bAdv )
                        {
                           DdeClientTransaction( NULL, 0, hConv, hszItem,
                                                 CF_TEXT, XTYP_ADVSTART, 
                                                 2000,  NULL );

                           bAdv = DdeGetLastError( ddeInst ) == DMLERR_NO_ERROR;
                        }    
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( hConv )
              {
                 if ( bAdv )
                    DdeClientTransaction( NULL, 0, hConv, hszItem,
                                          CF_TEXT, XTYP_ADVSTOP, 
                                          2000,  NULL );
                 DdeDisconnect( hConv );
              }

              DdeFreeStringHandle( ddeInst, hszService );
              DdeFreeStringHandle( ddeInst, hszTopic );
              DdeFreeStringHandle( ddeInst, hszItem );

              DdeUninitialize( ddeInst );
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


HDDEDATA CALLBACK DdemlCallback( UINT     uType,
                                 UINT     uFmt,	
                                 HCONV    hconv,	
                                 HSZ      hsz1,	
                                 HSZ      hsz2,	
                                 HDDEDATA hdata,	
                                 DWORD    dwData1,
                                 DWORD    dwData2 )
{
static char     szTime[128];
static CONVINFO convInfo;

   switch( uType )
   {
      case XTYP_ADVDATA : 
             DdeGetData( hdata, szTime, sizeof( szTime ), 0 );
             szTime[strlen(szTime)-2] = 0;

             convInfo.cb = sizeof( CONVINFO );
             DdeQueryConvInfo( hConv, QID_SYNC, &convInfo );

             SendMessage( (HWND)convInfo.hUser, LB_RESETCONTENT, 0, 0 );
             SendMessage( (HWND)convInfo.hUser, LB_ADDSTRING, 0, (LPARAM)szTime );
             break;
   }
   
   return( NULL );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
